//-----------------------------------------------------------------------------
// Copyright (C) 2008  PROCEQ SA, CH-8603 Schwerzenbach
//-----------------------------------------------------------------------------
// Package                : PqRemoteDemoDLL
// Environment            : WIN32, MFC
// Tested on              : - Studio 2008 (V9)
//-----------------------------------------------------------------------------
// Source Code Management : $Workfile: PqRemoteDemoDLLDlg.cpp $
//                          $Revision: 8 $ $Date: 21.01.15 15:56 $ $NoKeywords: $
//-----------------------------------------------------------------------------
// Description:
//   The application dialog that demonstrates the useage of the PqRemote DLL 
//   under C++. This file is the implementation of the dialog demonstrating 
//   the methods of the PqRemote DLL.
//   In here you can see how the DLL is loaded and used.
//
//-----------------------------------------------------------------------------
// History (date author comment)
// -------
// 23-Oct-2008  MAP Initial
// 05-Sep-2014  MAP Device Type can be edited (Platform)
// 16-Jan-2015  MAP Don't allow Serial for Platform
// 27-Oct-2017  MAP Support for 64 bit DLL
//-----------------------------------------------------------------------------


//=== Import ==================================================================
#include "stdafx.h"
#include "PqRemoteDemoDLL.h"
#include "PqRemoteDemoDLLDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif

// the size of the buffer allocated for PQ_SendCMD
#define MAX_REPLY_LEN 20480
// the size of the buffer allocated for PQ_Read
#define MAX_READ_LEN 20480
// ID of the timer
#define TIMER_ID 1      

//=== Class CAboutDlg declaration =============================================
// This dialog shows the information about this application 
class CAboutDlg : public CDialog
{
public:
	CAboutDlg();

// Dialog Data
	enum { IDD = IDD_ABOUTBOX };

protected:
// DDX/DDV support
  virtual void DoDataExchange(CDataExchange* pDX);    

// Implementation
	DECLARE_MESSAGE_MAP()
};


//=== Class CAboutDlg implementation ==========================================
//--- Constructor -------------------------------------------------------------
CAboutDlg::CAboutDlg() : CDialog(CAboutDlg::IDD)
{
}

//--- DDX/DDV support ---------------------------------------------------------
void CAboutDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
}

//--- Message Map -------------------------------------------------------------
BEGIN_MESSAGE_MAP(CAboutDlg, CDialog)
END_MESSAGE_MAP()



//=== Class CPqRemoteDemoDLLDlg implementation ================================
//--- Constructor -------------------------------------------------------------
CPqRemoteDemoDLLDlg::CPqRemoteDemoDLLDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CPqRemoteDemoDLLDlg::IDD, pParent),
  mDeviceHandle(0),
  mSendCMD(NULL),
  mRead(NULL),
  mCheckCompatibility(NULL),
  mConnect(NULL),
  mDisconnect(NULL),
  mReadTimeout(0),
  mhLib(NULL),
  mConnectTimeout(10000), 
  mPort(1),
  mTimerp(0)
  , mDeviceType(10)
{
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
}

//--- Destructor --------------------------------------------------------------
CPqRemoteDemoDLLDlg::~CPqRemoteDemoDLLDlg()
{
  // Stop the timer
  if (mTimerp)
  {
    KillTimer(mTimerp);
    mTimerp=0;
  }

  // Disconnect any left open device connection
  if(mDeviceHandle)
    (*mDisconnect)(mDeviceHandle);

  // Free the PqRemote DLL 
  if(mhLib)
    FreeLibrary(mhLib);
}


//--- DDX/DDV support ---------------------------------------------------------
void CPqRemoteDemoDLLDlg::DoDataExchange(CDataExchange* pDX)
{
  CDialog::DoDataExchange(pDX);
  DDX_Control(pDX, IDC_SENDCMD, mSendCmdText);
  DDX_Control(pDX, IDC_CMD_ANSWER, mCmdAnswer);
  DDX_Control(pDX, IDC_READ_RESULT, mReadText);
  DDX_Text(pDX, IDC_READ_TIMEOUT, mReadTimeout);
  DDX_Control(pDX, IDC_CONNECTBUTTON, mConnectButton);
  DDX_Control(pDX, IDC_DISCONNECTBUTTON, mDisconnectButton);
  DDX_Text(pDX, IDC_CONNECT_TIMEOUT, mConnectTimeout);
  DDX_Control(pDX, IDC_IPADDRESS, mIpAddress);
  DDX_Text(pDX, IDC_PORTNUMBER, mPort);
  DDV_MinMaxUInt(pDX, mPort, 1, 1024);
  DDV_MinMaxLong(pDX, mConnectTimeout, 0, 100000);
  DDX_Control(pDX, IDC_CHECKSCAN, mCheckScan);
  DDX_Control(pDX, IDC_COMBOINTERFACE, mComboInterface);
  DDX_Control(pDX, IDC_SENDBUTTON, mSendButton);
  DDX_Control(pDX, IDC_READBUTTON, mReadButton);
  DDX_Control(pDX, IDC_CHECKREAD, mCheckRead);
  DDX_Control(pDX, IDC_READ_TIMEOUT, mReadTimeoutText);
  DDX_Text(pDX, IDC_DEVICETYPE, mDeviceType);
	DDV_MinMaxByte(pDX, mDeviceType, 0, 32);
}

//--- Message Map -------------------------------------------------------------
BEGIN_MESSAGE_MAP(CPqRemoteDemoDLLDlg, CDialog)
	ON_WM_SYSCOMMAND()
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
  ON_BN_CLICKED(IDC_SENDBUTTON, &CPqRemoteDemoDLLDlg::OnBnClickedSendbutton)
  ON_BN_CLICKED(IDC_READBUTTON, &CPqRemoteDemoDLLDlg::OnBnClickedReadbutton)
  ON_BN_CLICKED(IDC_CONNECTBUTTON, &CPqRemoteDemoDLLDlg::OnBnClickedConnectbutton)
  ON_BN_CLICKED(IDC_DISCONNECTBUTTON, &CPqRemoteDemoDLLDlg::OnBnClickedDisconnectbutton)
  ON_BN_CLICKED(IDC_CHECKREAD, &CPqRemoteDemoDLLDlg::OnBnClickedCheckread)
  ON_WM_TIMER()
  ON_BN_CLICKED(IDC_CLEAR_READ, &CPqRemoteDemoDLLDlg::OnBnClickedClearRead)
  ON_EN_CHANGE(IDC_DEVICETYPE, &CPqRemoteDemoDLLDlg::OnEnChangeDevicetype)
END_MESSAGE_MAP()


//--- Dialog initialization. Loads the DLL, too -------------------------------
BOOL CPqRemoteDemoDLLDlg::OnInitDialog()
{
  // use default read timeout defined in the DLL
  mReadTimeout = -1;

  // make sure sizeof long is 4 bytes. Only checked in debug builds
  ASSERT(sizeof(long) == 4);

  // Load the dll
#ifdef _WIN64
  ASSERT(sizeof(void *) == 8);
  ASSERT(sizeof(char *) == 8);

  mhLib = LoadLibrary(_T("PqRemoteX64.dll"));
  if (mhLib == NULL)
  {
    AfxMessageBox(_T("PqRemoteX64.dll not found.\r\nMake sure its in PATH or in the same folder as this .exe"));
    EndDialog(-1);
    return TRUE;
}
#else
  ASSERT(sizeof(void *) == 4);
  ASSERT(sizeof(char *) == 4);

  mhLib = LoadLibrary(_T("PqRemote.dll"));
  if (mhLib == NULL)
  {
    AfxMessageBox(_T("PqRemote.dll not found.\r\nMake sure its in PATH or in the same folder as this .exe"));
    EndDialog(-1);
    return TRUE;
  }
#endif


  //--- Get the pointer to the exported functions of the dll
  mCheckCompatibility = (PQ_LPCHECKCOMPATIBILITY)GetProcAddress(mhLib, "PQ_CheckCompatibility"); 
  mConnect = (PQ_LPCONNECTDEVICE)GetProcAddress(mhLib, "PQ_ConnectDevice"); 
  mDisconnect = (PQ_LPDISCONNECTDEVICE)GetProcAddress(mhLib, "PQ_DisconnectDevice");
  mSendCMD = (PQ_LPSENDCMD)GetProcAddress(mhLib, "PQ_SendCMD");
  mRead = (PQ_LPREAD)GetProcAddress(mhLib, "PQ_Read");

  // Everything found?
  if(mCheckCompatibility == NULL || mConnect == NULL || mDisconnect == NULL || mSendCMD == NULL || mRead == NULL)
  {
    FreeLibrary(mhLib);
    EndDialog(-1);
    return TRUE;
  }
  // Is the Version of the DLL compatible with this application?
  unsigned long res=(*mCheckCompatibility)(PQ_REMOTE_INTERFACE_MAIN_VERSION,
                            PQ_REMOTE_INTERFACE_SIDE_VERSION);
  if(res > 1)
  {
    AfxMessageBox(_T("Incompatible PqRemote DLL Version"));

    FreeLibrary(mhLib);
    EndDialog(-1);
    return TRUE;
  }

  // Base class initialization
	CDialog::OnInitDialog();

  //--- Initialize the controls
  mComboInterface.ResetContent();
  mComboInterface.AddString(_T("Serial"));
  mComboInterface.AddString(_T("USB"));
  mComboInterface.AddString(_T("Ethernet"));
  mComboInterface.SetCurSel(1);
  mCheckScan.SetCheck(BST_UNCHECKED);
  mCheckRead.SetCheck(BST_UNCHECKED);
  mIpAddress.SetAddress(192,168,0,204);
  mDisconnectButton.EnableWindow(FALSE);
  mConnectButton.EnableWindow();
  mSendButton.EnableWindow(FALSE);
  mReadButton.EnableWindow(FALSE);
  mReadTimeoutText.EnableWindow(FALSE);
  mCheckRead.EnableWindow(FALSE);


	// Add "About..." menu item to system menu.
	// IDM_ABOUTBOX must be in the system command range.
	ASSERT((IDM_ABOUTBOX & 0xFFF0) == IDM_ABOUTBOX);
	ASSERT(IDM_ABOUTBOX < 0xF000);

	CMenu* pSysMenu = GetSystemMenu(FALSE);
	if (pSysMenu != NULL)
	{
		CString strAboutMenu;
		strAboutMenu.LoadString(IDS_ABOUTBOX);
		if (!strAboutMenu.IsEmpty())
		{
			pSysMenu->AppendMenu(MF_SEPARATOR);
			pSysMenu->AppendMenu(MF_STRING, IDM_ABOUTBOX, strAboutMenu);
		}
	}

	// Set the icon for this dialog.  The framework does this automatically
	//  when the application's main window is not a dialog
	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon

  // Use 11 point regular monospace font in text answers from device
  CFont * pf = mReadText.GetFont();
  if(pf != NULL)
  {
    LOGFONT lf;
    CPoint      aPoint;
    HDC hdc = (mReadText.GetDC())->GetSafeHdc();

    aPoint.y = (11 * GetDeviceCaps(hdc,LOGPIXELSY))/72;
    ::DPtoLP(hdc,&aPoint,1);
    memset(&lf,0,sizeof(lf));
    lf.lfHeight = aPoint.y;
    lf.lfWeight = FW_REGULAR;
    lf.lfOutPrecision = OUT_TT_PRECIS; // Truetype font
    lf.lfQuality = PROOF_QUALITY;
    lf.lfPitchAndFamily = FIXED_PITCH | FF_MODERN;
    mFont.CreateFontIndirect(&lf); 
    mReadText.SetFont(&mFont); 
    mCmdAnswer.SetFont(&mFont);
  }
  UpdateData(0);

	return TRUE;  // return TRUE  unless you set the focus to a control
}

//--- System command handler --------------------------------------------------
void CPqRemoteDemoDLLDlg::OnSysCommand(UINT nID, LPARAM lParam)
{
	if ((nID & 0xFFF0) == IDM_ABOUTBOX)
	{
		CAboutDlg dlgAbout;
		dlgAbout.DoModal();
	}
	else
	{
		CDialog::OnSysCommand(nID, lParam);
	}
}


//--- Paint the dialog --------------------------------------------------------
// If you add a minimize button to your dialog, you will need the code below
//  to draw the icon.  For MFC applications using the document/view model,
//  this is automatically done for you by the framework.
void CPqRemoteDemoDLLDlg::OnPaint()
{
	if (IsIconic())
	{
		CPaintDC dc(this); // device context for painting

		SendMessage(WM_ICONERASEBKGND, reinterpret_cast<WPARAM>(dc.GetSafeHdc()), 0);

		// Center icon in client rectangle
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Draw the icon
		dc.DrawIcon(x, y, m_hIcon);
	}
	else
	{
		CDialog::OnPaint();
	}
}

//-----------------------------------------------------------------------------
// The system calls this function to obtain the cursor to display while the
// user drags the minimized window.
HCURSOR CPqRemoteDemoDLLDlg::OnQueryDragIcon()
{
	return static_cast<HCURSOR>(m_hIcon);
}


//-----------------------------------------------------------------------------
// Demonstrates the useage of the PQ_SendCMD method from PqRemote
void CPqRemoteDemoDLLDlg::OnBnClickedSendbutton()
{
  // For Textconversion
  USES_CONVERSION;                      

  // allocate and initialize the reply buffer
  char reply[MAX_REPLY_LEN+1];
  memset(reply,0,MAX_REPLY_LEN+1);

  // Clear the PQ_SendCMD output 
  mCmdAnswer.SetWindowText(_T(""));
  mCmdAnswer.RedrawWindow();

  // Get the command to send to the device
  CString cmd;
  mSendCmdText.GetWindowText(cmd);
  // Send the command to the device and receive the reply if any in default time
  long replyLen = (*mSendCMD)(mDeviceHandle,T2A(cmd),reply,MAX_REPLY_LEN,-1);

  CString s(_T("*** PQ_SendCMD ok. Answer from device see below. ***\r\n\r\n"));

  // Error occurred?
  if(replyLen < 0)
    s = _T("*** PQ_SendCMD failed. ***");
  // Device received the command but no reply was needed
  else if(replyLen == 0)
    s += _T("<no reply>");
  // Device received the command and answered to it.
  else
    s += reply;

  // Output the result to the dialog
  mCmdAnswer.SetWindowText(s);
}


//-----------------------------------------------------------------------------
// Demonstrates the useage of the PQ_Read method from PqRemote
void CPqRemoteDemoDLLDlg::OnBnClickedReadbutton()
{
  // allocate and initialize the read buffer
  char buffer[MAX_READ_LEN];
  memset(buffer,0,MAX_READ_LEN);

  // Display the wait cursor
  CWaitCursor c;
  UpdateData();

  // Tell the user that we are currently reading from the device
  mReadText.SetWindowText(_T("Reading..."));
  mReadText.RedrawWindow();

  // Read upto MAX_READ_LEN bytes or mReadTimeout occurred
  long bytesRead = (*mRead)(mDeviceHandle,buffer,MAX_READ_LEN-1,mReadTimeout);

  CString s(_T("*** Timeout occurred and no byte was read. ***"));
  // We received an answer.
  if(bytesRead > 0)
    s = buffer;

  // EOL and in Echo mode? NOTE: the <HELP> command could be an exception
  if(s.Find(_T("\r\n")) < 0)
  {
    // no, insert a newline
    s.Replace(_T("\r"),_T("\r\n"));
  }
  // Display the bytes read...
  mReadText.SetWindowText(s);
  // ...and move to the last line
  mReadText.LineScroll(mReadText.GetLineCount());
}


//-----------------------------------------------------------------------------
// Demonstrates the useage of the PQ_ConnectDevice method from PqRemote.
// NOTE:
//   - Device type is set to Equotip3 in this demo.
//   - Parameters not needed for the selected interface are ignored.
//   - Ethernet connections can take a long time (>25s) to establish, therefore 
//     if mConnectTimeout is too short, PQ_ConnectDevice()
//     returns with handle 0 while the Thread in the DLL is still working (it
//     processes jobs sequentially and waits until the socket operation has 
//     finished. Therefore if you call PQ_ConnectDevice again too early (but 
//     this time with correct ip address), it might timeout again.
void CPqRemoteDemoDLLDlg::OnBnClickedConnectbutton()
{
  UpdateData();
  byte f0, f1, f2, f3;
  char ip[24];
  ip[23] = 0;

  // generate the ip address as string
  mIpAddress.GetAddress(f0,f1,f2,f3);
  sprintf_s(ip,"%d.%d.%d.%d",f0,f1,f2,f3);

  // Already an open connection? close it!
  if(mDeviceHandle != 0)
  {
    (*mDisconnect)(mDeviceHandle);
    mDeviceHandle = 0;
  }
  // Serial port not supported on platform
  if(mDeviceType == 10 && (mComboInterface.GetCurSel()==0))
  {
    AfxMessageBox(_T("Serial Interface not supported on platform device"));
    mComboInterface.SetCurSel(1);
    return;
  }
  // Connect to the device 
  mDeviceHandle = (*mConnect)(mDeviceType,1<<mComboInterface.GetCurSel(),mCheckScan.GetCheck()== BST_CHECKED ? true:false,
                              mPort,ip,mConnectTimeout);

  // No device found or time out occurred?
  if(mDeviceHandle == 0)
  {
    AfxMessageBox(_T("Connection failed"));
    return;
  }
  // Device found --> Change controls
  mDisconnectButton.EnableWindow();
  mConnectButton.EnableWindow(FALSE);
  mSendButton.EnableWindow();
  mReadButton.EnableWindow();
  mReadTimeoutText.EnableWindow();
  mCheckRead.EnableWindow();
}


//-----------------------------------------------------------------------------
// Demonstrates the useage of the PQ_DisconnectDevice method from PqRemote.
void CPqRemoteDemoDLLDlg::OnBnClickedDisconnectbutton()
{
  // Opened device?
  if(mDeviceHandle != 0)
  {
    // disconnect and reset the handle 
    (*mDisconnect)(mDeviceHandle);
    mDeviceHandle = 0;
  }

  // Change the controls
  mReadTimeoutText.EnableWindow(FALSE);
  mDisconnectButton.EnableWindow(FALSE);
  mConnectButton.EnableWindow();
  mSendButton.EnableWindow(FALSE);
  mReadButton.EnableWindow(FALSE);
  mReadText.SetWindowTextW(_T(""));
  mCmdAnswer.SetWindowTextW(_T(""));
  mSendCmdText.SetWindowTextW(_T(""));
  mCheckRead.SetCheck(BST_UNCHECKED);
  mCheckRead.EnableWindow(FALSE);

  // Stop the timer if continuous read was on
  if (mTimerp)
  {
    KillTimer(mTimerp);
    mTimerp=0;
  }
}


//-----------------------------------------------------------------------------
// The continuous read checkbox was clicked...
void CPqRemoteDemoDLLDlg::OnBnClickedCheckread()
{
  if(mCheckRead.GetCheck()== BST_CHECKED)
  {
    // Continuous read is enabled, disable other controls
    mReadButton.EnableWindow(FALSE);
    mReadTimeoutText.EnableWindow(FALSE);

    // Create the timer so a read operation is started every 200ms
    mTimerp = SetTimer(TIMER_ID, 200, 0);
    if (!mTimerp)
    {
      ASSERT(false);
    }
  }
  else
  {
    // Continuous read is disabled, enable other controls and kill the timer.
    mReadButton.EnableWindow();
    mReadTimeoutText.EnableWindow();
    if (mTimerp)
    {
      KillTimer(mTimerp);
      mTimerp=0;
    }
  }
}


//-----------------------------------------------------------------------------
// The continuous read timer reads from the device and returns immediately if
// no byte is in the buffer
void CPqRemoteDemoDLLDlg::OnTimer(UINT_PTR nIDEvent)
{
  // Is it our timer and is a connection to the device available?
  if (nIDEvent == TIMER_ID && mDeviceHandle != NULL)
  {
    // allocate and initialize the read buffer
    char buffer[MAX_READ_LEN];
    memset(buffer,0,MAX_READ_LEN);

    // get the text already received in previous read calls
    CString s,t; 
    mReadText.GetWindowText(s);

    // Read from the device
    long bytesRead = (*mRead)(mDeviceHandle,buffer,MAX_READ_LEN-1,0);

    // Any new byte received?
    if(bytesRead > 0)
    {
      t = buffer;
      // EOL and in Echo mode ?
      if(t.Find(_T("\r\n")) < 0)
      {
        // no, insert a newline
        t.Replace(_T("\r"),_T("\r\n"));
      }
      // Update the output and scroll to the end
      mReadText.SetWindowText(s+t);
      mReadText.LineScroll(mReadText.GetLineCount());
    }
  }
  CDialog::OnTimer(nIDEvent);
}

//-----------------------------------------------------------------------------
// Clear the output of Pq_Read
void CPqRemoteDemoDLLDlg::OnBnClickedClearRead()
{
  mReadText.SetWindowText(_T(""));
}

//-----------------------------------------------------------------------------
// Change interface to a valid one in case of platform device
void CPqRemoteDemoDLLDlg::OnEnChangeDevicetype()
{
  UpdateData();
  if(mDeviceType == 10 && (mComboInterface.GetCurSel()==0))
  {
    mComboInterface.SetCurSel(1);
  }
}
